const { Clutter, GObject, St } = imports.gi;
const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;

class Extension {
    constructor(uuid) {
        this._uuid = uuid;
        this._counter = 0;

    }

    enable() {
        this._indicator = new Indicator(this);
        Main.panel.addToStatusArea(this._uuid, this._indicator, 4, 'left');
    }

    disable() {
        this._indicator.destroy();
        this._indicator = null;
    }

    incrementCounter() {
        this._counter++;
        this._indicator.updateLabel();
    }

    decrementCounter() {
        this._counter--;
        if (this._counter < 0) {
            this._counter = 0;
        }
        this._indicator.updateLabel();
    }

    resetCounter() {
        this._counter = 0;
        this._indicator.updateLabel();
    }

    getCounter() {
        return this._counter;
    }
}

const Indicator = GObject.registerClass(
class Indicator extends PanelMenu.Button {
    _init (extension) {
        super._init(0.0, 'Toggle Button');

        this._extension = extension;

        this._label = new St.Label({
            text: ` COUNT: ${extension.getCounter()} | `,
            y_align: Clutter.ActorAlign.CENTER,
        });
        this.add_child(this._label);
        this.connect('button-press-event', this._onButtonPress.bind(this));
    }

    _onButtonPress(actor, event) {
        switch (event.get_button()) {
            case Clutter.BUTTON_PRIMARY: // Left click
                this._extension.incrementCounter();
                break;
            case Clutter.BUTTON_SECONDARY: // Right click
                this._extension.decrementCounter();
                break;
            case Clutter.BUTTON_MIDDLE: // Middle click
                this._extension.resetCounter();
                break;
            default:
                break;
        }
        return Clutter.EVENT_PROPAGATE;
    }

    updateLabel() {
        this._label.set_text(` COUNT: ${this._extension.getCounter()} | `);
    }
});

function init(meta) {
    return new Extension(meta.uuid);
}

